

// defines to be put in .h file

#define DS18B20_9_BIT_UNDF       ((1<<0)|(1<<1)|(1<<2))

////////////////////////////////////////////////////////////////////////////
// GLOBAL VARIABLES
//
unsigned char ROM[8];           // ROM Bit
unsigned char lastDiscrep = 0;  // last discrepancy
unsigned char doneFlag = 0;     // Done flag
unsigned char FoundROM[6][8];   // table of found ROM codes
unsigned char numROMs;
unsigned char dowcrc;

// for CRC = X8 + X5 + X4 + 1
char const dscrc_table[] = {
0, 94,188,226, 97, 63,221,131,194,156,126, 32,163,253, 31, 65,
157,195, 33,127,252,162, 64, 30, 95, 1,227,189, 62, 96,130,220,
35,125,159,193, 66, 28,254,160,225,191, 93, 3,128,222, 60, 98,
190,224, 2, 92,223,129, 99, 61,124, 34,192,158, 29, 67,161,255,
70, 24,250,164, 39,121,155,197,132,218, 56,102,229,187, 89, 7,
219,133,103, 57,186,228, 6, 88, 25, 71,165,251,120, 38,196,154,
101, 59,217,135, 4, 90,184,230,167,249, 27, 69,198,152,122, 36,
248,166, 68, 26,153,199, 37,123, 58,100,134,216, 91, 5,231,185,
140,210, 48,110,237,179, 81, 15, 78, 16,242,172, 47,113,147,205,
17, 79,173,243,112, 46,204,146,211,141,111, 49,178,236, 14, 80,
175,241, 19, 77,206,144,114, 44,109, 51,209,143, 12, 82,176,238,
50,108,142,208, 83, 13,239,177,240,174, 76, 18,145,207, 45,115,
202,148,118, 40,171,245, 23, 73, 8, 86,180,234,105, 55,213,139,
87, 9,235,181, 54,104,138,212,149,203, 41,119,244,170, 72, 22,
233,183, 85, 11,136,214, 52,106, 43,117,151,201, 74, 20,246,168,
116, 42,200,150, 21, 75,169,247,182,232, 10, 84,215,137,107, 53};


// function prototypes
int ow_reset(void);
int read_bit(void);
void write_bit(int bitval);
int read_byte(void);
void write_byte(char val);
void FindDevices(void);
int First(void);
char ow_crc( unsigned char x);
char Next(void);
void Read_Temperature(void);
void Read_ScratchPad(void);
void Read_ROMCode(void);
char Send_MatchRom(void);
void DS_main();




//****************************************************************************
// OW_RESET - performs a reset on the one-wire bus and
// returns the presence detect.
//
int ow_reset(void){

    char presence;
    set_tris_j(PORT_J_MASK);
    output_low(PIN_J5);         // pull DQ line low
    delay_us(480);              // leave it low for 480us
    output_high(PIN_J5);        // allow line to return high
    output_float(PIN_J5);       // Set pin to input mode
    delay_us(71);               // wait for presence
    presence = input(PIN_J5);   // get presence signal
    delay_us(420);              // wait for end of timeslot
    if(input(PIN_J5) == 0)      // no presence detect
        presence = 2;           // short circuit detected
    return(presence);           // presence signal returned
}                               // 0 = presence, 1 = open or no part, 2 = short circuit


//////////////////////////////////////////////////////////////////
// READ_BIT - reads a bit from the one-wire bus.
//
int read_bit(void)
{
    unsigned char read_bit;
    set_tris_j(PORT_J_MASK);
    output_low(PIN_J5);      // pull DQ low to start timeslot
    output_high(PIN_J5);      // then return high
    output_float(PIN_J5);      // Set pin to input mode
    delay_us(14);// 15us
    read_bit = input(PIN_J5);
    return(read_bit);     // return value of DQ line
}



//////////////////////////////////////////////////////////////////
// WRITE_BIT - writes a bit to the one-wire bus, passed in bitval.
//
void write_bit(int bitval){

    set_tris_j(PORT_J_MASK);
    output_low(PIN_J5);      // pull DQ low to start timeslot

    if(bitval==1){
        output_high(PIN_J5);  // return DQ high if write 1
    }
    delay_us(103);                  // hold value for remainder of timeslot
    output_high(PIN_J5);      // then return high
}   // Delay provides 16us per loop, plus 24us. Therefore delay(5) = 104us



///////////////////////////////////////////////////////////////////
// READ_BYTE - reads a byte from the one-wire bus.
//
int read_byte(void){

    int i;
    int value = 0;
    for (i=0; i<8; i++)
    {
        if(read_bit()) value |= 0x01<<i;
        // reads byte in, one byte at a time and then
        // shifts it left
        delay_us(119); // wait for rest of timeslot
    }
    return(value);
}



///////////////////////////////////////////////////////
// WRITE_BYTE - writes a byte to the one-wire bus.
//
void write_byte ( char val)
{

    int i;
    int temp;

    for (i = 0 ; i < 8 ; i++ )     // writes byte, one bit at a time
    {
        temp = val >> i;    // shifts val right 'i' spaces
        temp &= 0x01;       // copy that bit to temp
        write_bit(temp);    // write bit in temp into
    }
    delay_us(103);
}



// FIND DEVICES
void FindDevices(void){
    unsigned char m;
    if(ow_reset() == 0) //Begins when a presence is detected
    {
        //printf("\r\nRESET OK\r\n");
        if(First()) //Begins when at least one part is found
        {
            numROMs=0;
            do
            {
                numROMs++;
                for(m=0;m<8;m++)
                {
                    FoundROM[numROMs][m]=ROM[m]; //Identifies ROM
                    // number on found device
                }

                printf(put_COM1, "\nROM CODE = %02X%02X%02X%02X%02X%02X%02X%02X\n",
                    FoundROM[5][7],FoundROM[5][6],FoundROM[5][5],FoundROM[5][4],
                    FoundROM[5][3],FoundROM[5][2],FoundROM[5][1],FoundROM[5][0]);

            }while (Next() && (numROMs<10));
            // (Continues until no additional devices are found)
        }else{
            printf(put_COM1, "\r\nNO PART FOUND\r\n");
        }
    }
}



// FIRST
// The First function resets the current state of a ROM search and calls
// Next to find the first device on the 1-wire bus.
//
int First(void){

    lastDiscrep = 0;    // reset the rom search last discrepancy global

    doneFlag = FALSE;
    return Next();      // call Next and return its return value
}




// NEXT
// The Next function searches for the next device on the 1-wire bus. If
// there are no more devices on the 1-wire then false is returned.
//
char Next(void){

    unsigned char m = 1; // ROM Bit index
    unsigned char n = 0; // ROM Byte index
    unsigned char k = 1; // bit mask
    unsigned char x = 0;
    unsigned char discrepMarker = 0; // discrepancy marker
    unsigned char g;        // Output bit
    unsigned char nxt;      // return value
    int flag;

    nxt = FALSE;            // set the next flag to false
    dowcrc = 0;             // reset the dowcrc

    flag = ow_reset();      // reset the 1-wire

    if(flag || doneFlag)    // no parts -> return false
    {
        lastDiscrep = 0;    // reset the search
        return (FALSE);
    }

    // send SearchROM command
    write_byte(0xf0);

    // for all eight bytes
    do
    {
        x = 0;
        if(read_bit()==1) x = 2;
        delay_us(119);
        if(read_bit()==1) x |= 1; // and its complement
        if(x ==3) // there are no devices on the 1-wire
            break;
        else
        {
            if(x>0) // all devices coupled have 0 or 1
            g = x>>1; // bit write value for search
            else
            {
                // if this discrepancy is before the last
                // discrepancy on a previous Next then pick
                // the same as last time
                if(m<lastDiscrep)
                    g = ((ROM[n] & k) > 0);
                else // if equal to last pick 1
                    g = (m == lastDiscrep); // if not then pick 0
                    // if 0 was picked then record
                    // position with mask k
                if (g==0) discrepMarker = m;
            }
            if(g==1) // isolate bit in ROM[n] with mask k
                ROM[n] |= k;
            else
                ROM[n] &= ~k;
            write_bit(g);       // ROM search write
            m++;                // increment bit counter m
            k = k<<1;           // and shift the bit mask k
            if(k==0)            // if the mask is 0 then go to new ROM
            {                   // byte n and reset mask
                ow_crc(ROM[n]); // accumulate the CRC
                n++; k++;
            }
        }
    }while(n<8);                //loop until through all ROM bytes 0-7

    if(m<65||dowcrc)            // if search was unsuccessful then
        lastDiscrep=0;              // reset the last discrepancy to 0
    else
    {
        // search was successful, so set lastDiscrep,
        // lastOne, nxt
        lastDiscrep = discrepMarker;
        doneFlag = (lastDiscrep==0);
        nxt = TRUE; // indicates search is not complete yet, more
        // parts remain
    }
    return nxt;
}



//////////////////////////////////////////////////////
// ONE WIRE CRC
// uses CRC = X8 + X5 + X4 + 1
char ow_crc( unsigned char x){

    dowcrc = dscrc_table[dowcrc^x];
    return dowcrc;
}


//////////////////////////////////////////////////////
//
//
void Read_Temperature(void){

    int get[10];
    int k;
    signed long temp;
    long meas;

    union {
    int tmp_in[2];
    long tmp_w;
    } value;

    k = ow_reset();

    if(k == 0)
    {

        ow_reset();
        write_byte(0xCC); // Skip ROM
        write_byte(0x44); // Start Conversion
        delay_us(103);
        //        delay_ms(500);
        ow_reset();
        write_byte(0xCC); // Skip ROM
        write_byte(0xBE); // Read Scratch Pad

        for (k=0; k<9; k++){
            get[k] = read_byte();
        }

        // Initialize CRC value
        dowcrc = 0;
        // calculate CRC
        for (k=0; k<8; k++){
            ow_crc(get[k]);
        }
        // Verify CRC values
        if(dowcrc == get[8])
            printf(put_COM1, "\n CRC OK\n");
        else
            printf(put_COM1, "\n CRC FAIL\n");

        printf(put_COM1, "\n ScratchPAD DATA = %2X %2X%2X %2X%2X %2X%2X %2X%2X\n",
            get[8],get[7],get[6],get[5],get[4],get[3],get[2],get[1],get[0]);

         value.tmp_in[0]=get[0]; //LSB
         value.tmp_in[1]=get[1]; //MSB

        // Check for sign bit.
        if(value.tmp_w > 32767){
            temp = (signed long) value.tmp_w - 65536;
        }else
            temp = (signed long) value.tmp_w;

        // For DS18B20 in 12bit mode
        printf(put_COM1, "\n T = %3.2f degrees C, ", (float)temp / 16.0 );
        // C to F conversion
        // DS1820: [(T*18)+640]/20
        temp = (temp * 9) + 2560;
        printf(put_COM1, " %3.2f degrees F\n", (float)temp / 80.0 );     // print temp. F
    }
    else if(k == 2)
        printf(put_COM1, "\n Short Circuit Detected\n");
    else if (k == 1)
        printf(put_COM1, "\n Open Circuit Detected\n");
}



///////////////////////////////////////////////////
//
//
char Read_power_supply(){

    int pstat=0;

    ow_reset();
    write_byte(0xCC); // Skip ROM
    write_byte(0xB4); // Read Power Supply Status
    pstat = read_bit() & 0x01;
    printf(put_COM1, "\n READ Power Supply Status = %u \n", pstat);
    return pstat;
}


///////////////////////////////////////////////////////
//
//
void Read_ScratchPad(void)
{
    int j;
    char pad[10];

    printf(put_COM1, "\nReading ScratchPad Data\n");
    write_byte(0xBE);

    for (j=0;j<9;j++){pad[j]=read_byte();}

    printf(put_COM1, "\n ScratchPAD DATA = %2X %2X%2X %2X%2X %2X%2X %2X%2X\n",
            pad[8],pad[7],pad[6],pad[5],pad[4],pad[3],pad[2],pad[1],pad[0]);

}



void Read_ROMCode(void)
{
    int n;
    char dat[9];

    printf(put_COM1, "\nReading ROM Code\n");
    ow_reset();
    write_byte(0x33);
    for (n=0;n<8;n++){dat[n]=read_byte();}
    printf(put_COM1, "\n ROM Code = %2X%2X %2X%2X %2X%2X %2X%2X\n",
            dat[7],dat[6],dat[5],dat[4],dat[3],dat[2],dat[1],dat[0]);
}



// Perform Match ROM
//
char Send_MatchRom(void)
{
    unsigned char i;

    if(ow_reset()) return false;

    write_byte(0x55); // match ROM

    for(i=0;i<8;i++)
    {
        write_byte(FoundROM[numROMs][i]); //send ROM code
    }
    return true;
}

///////////////////////////////////////////////////////////////////////////////////////

// 1wiretalk.c -- Functions for the Dallas Semiconductor DS18x20/DS1822
// Two-Wire Temperature Sensor
// Designed for 8051 microcontrollers
// This code was developed using the DS5000/DS2251T
// Please note that 128K RAM size is required to run this program.

/////////////////////////BEGIN MAIN PROGRAM//////////////////////////////

void DS_main()
{
    int done = 0;
    int Select_Type; /* Function variable */

    printf (put_COM1, "\n");
    printf (put_COM1, " Dallas Semiconductor - Systems Extension\n");
    printf (put_COM1, " Adapted from AN162\n");
    printf (put_COM1, " Source for DS1820 Temperature Reading and\n");
    printf (put_COM1, " Search ROM code.\n");
    printf (put_COM1, " Updated Code April, 2006 \n");
    printf (put_COM1, " [C Program for PIC8722 Microcontroller]");
    printf (put_COM1, "\n");
    printf (put_COM1, "\n***********************************************\n");
    printf (put_COM1, " Select Menu Option\n");
    printf (put_COM1, " 1. One-Wire Reset\n");
    printf (put_COM1, " 2. Read ROM Code of Single Device On Net\n");
    printf (put_COM1, " 3. Perform Search ROM\n");
    printf (put_COM1, " 4. Read Scratch PAD\n");
    printf (put_COM1, " 5. Read Temperature\n");
    printf (put_COM1, " 6. Find All Devices\n");
    printf (put_COM1, " 7. Read Power Supply Status\n");
    printf (put_COM1, " 8. Return to Command System\n");
    printf (put_COM1, "\n\n");

    /*----------------------------------------------------------------------*/

    disable_interrupts(INT_RDA);

    do
    {
         while(!kbhit()) restart_wdt();
         Select_Type = toupper(getch()); /* get variable to start */
         disable_interrupts(GLOBAL);
         switch(Select_Type)
         {
         case '1':   printf (put_COM1, "\n 1. Sent 1-Wire Reset\n");
                     ow_reset();
                     break;
         case '2':   printf (put_COM1, "\n 2. Read ROM Code of Single Device On Net\n");
                     ow_reset();
                     Read_ROMCode();
                     break;
         case '3':   printf(put_COM1, "\n 3. Performing Search ROM\n");
                     ow_reset();
                     First();
                     printf(put_COM1, "\nROM CODE = %02X%02X %02X%02X %02X%02X %02X%02X\n",
                         FoundROM[5][7],FoundROM[5][6],FoundROM[5][5],FoundROM[5][4],
                         FoundROM[5][3],FoundROM[5][2],FoundROM[5][1],FoundROM[5][0]);
                     break;
         case '4':   printf (put_COM1, "\n 4. Read Scratch PAD\n");
                     ow_reset();
                     write_byte(0xCC); // Skip ROM
                     Read_ScratchPad();
                     break;
         case '5':   printf (put_COM1, "\n 5. Read Temperature\n");
                     Read_Temperature(); //initiates a temperature reading
                     break;
         case '6':   printf (put_COM1, "\n 6. Find All Devices\n");
                     ow_reset();
                     FindDevices();
                     break;
         case '7':   Read_power_supply();
                     break;
         case '8':   done = TRUE;
                     break;
         default:    printf ("\n Typo: Select Another Menu Option\n");
                     break;
         }; /* end switch*/
         enable_interrupts(GLOBAL);
    } while (!done); /* Loop forever */

    enable_interrupts(INT_RDA);

    //////////////////////////////////

}









